#!/usr/bin/env python

import json
import pathlib

import scrapy

import util.helpers


class WikipediaXhrSpider(scrapy.Spider):
    name = 'wikipedia_xhr'
    allowed_domains = ['wikipedia.org']
    custom_settings = {
        'FEEDS': {
            pathlib.Path('..', 'data', f'{name}.csv'): {
                'format': 'csv',
                'fields': ['title', 'extract'],
            },
        },
    }

    def start_requests(self):
        root_article = 'XMLHttpRequest'
        yield scrapy.Request(f'https://en.wikipedia.org/wiki/{root_article}',
                             callback=self.find_linked_articles,
                             cb_kwargs={'root_article': root_article})

    def find_linked_articles(self, response, root_article):
        for s in response.xpath('//a/@href').re(r'^/wiki/(.+)'):
            # The requests to the current page, the home page,
            # and the meta pages are discarded
            if (
                    not s.startswith(root_article)
                    and not s.startswith('Main_Page')
                    and not any(s.startswith(f'{x}:') for x in [
                        'Media',
                        'Special',
                        'Talk',
                        'User',      'User_talk',
                        'Wikipedia', 'Wikipedia_talk',
                        'File',      'File_talk',
                        'MediaWiki', 'MediaWiki_talk',
                        'Template',  'Template_talk',
                        'Help',      'Help_talk',
                        'Category',  'Category_talk',
                        'Portal',    'Portal_talk',
                        'Draft',     'Draft_talk',
                        'TimedText', 'TimedText_talk',
                        'Module',    'Module_talk',
                    ])
            ):
                yield response.follow(f'/api/rest_v1/page/summary/{s}',
                                      callback=self.parse_article_snippet)

    def parse_article_snippet(self, response):
        try:
            response_data = json.loads(response.text)
        except json.JSONDecodeError:
            self.logger.error('Failed to parse response text as JSON')
            return

        yield {
            k: util.helpers.normalize_space(util.helpers.jsonpath_find(
                response_data, v
            )) for k, v in [
                ('title', 'title'),
                ('extract', 'extract'),
            ]
        }
